/******************************************************************************
 * @file     iap.c
 * @brief    IAP main routine.
 * @version  1.0.0
 * @date     18, October, 2012
 *
 * @note
 * Copyright (C) 2012-2014 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#include <stdio.h>
#include "NUC1xx.h"
#include "Driver\DrvUART.h"
#include "Driver\DrvSYS.h"



#define USER_AP_MAX_SIZE		(50*1024)

#define USER_AP_ENTRY			(0)
#define ISP_CODE_ENTRY			(50*1024)
#define LD_BOOT_CODE_ENTRY		0x100000
#define PAGE_SIZE   			512


typedef void (FUNC_PTR)(void);


extern uint32_t ApImage1Base, ApImage1Limit; 
extern uint32_t ApImage2Base, ApImage2Limit;

static int  load_image_to_flash(uint32_t image_base, uint32_t image_limit, uint32_t flash_addr, uint32_t max_size);

__asm spChange(uint32_t _sp)
{
	MSR MSP, r0
	BX lr
}

/*---------------------------------------------------------------------------------------------------------*/
/*  Main Function									                                           			   */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main (void)
{
	STR_UART_T sParam;
	int  			u8Item;
	FUNC_PTR  		*func;
	volatile int  	loop;
	
	UNLOCKREG();
    SYSCLK->PWRCON.XTL12M_EN = 1;
    while(SYSCLK->CLKSTATUS.XTL12M_STB == 0);
	  
	/* Set UART Pin */
	outpw(&SYS->GPBMFP, inpw(&SYS->GPBMFP) | (0x3<<(0)));
	
	/* UART Setting */
    sParam.u32BaudRate 		= 115200;
    sParam.u8cDataBits 		= DRVUART_DATABITS_8;
    sParam.u8cStopBits 		= DRVUART_STOPBITS_1;
    sParam.u8cParity 		= DRVUART_PARITY_NONE;
    sParam.u8cRxTriggerLevel= DRVUART_FIFO_1BYTES;

	/* Set UART Configuration */
 	if(DrvUART_Open(UART_PORT0,&sParam) != E_SUCCESS)
	{
		printf("UART0 open failed\n");
		return FALSE;
	}
  
    printf("\n\n");
    printf("+--------------------------------------------------+\n");
    printf("|         IAP program running on APROM 50K         |\n");
    printf("+--------------------------------------------------+\n");

	/* Enable ISP function */
    FMC->ISPCON.ISPEN = 1;

    do
    {      
    	printf("\n\n\n");
		printf("+-------------------------------------------+\n");
		printf("|               Program Select              |\n");
		printf("+-------------------------------------------|\n");
		printf("| [0] Run LD boot program                   |\n");
		printf("| [1] Run user application                  |\n");
		printf("| [2] Load and run new user application 1   |\n");
		printf("| [3] Load and run new user application 2   |\n");
		printf("+-------------------------------------------+\n");
		printf("Please select...");
		u8Item = getchar();
		printf("%c\n", u8Item);
		
 		switch (u8Item)
		{
			case '0':
				func = 	(FUNC_PTR *)(*(uint32_t *)(LD_BOOT_CODE_ENTRY+4));
				printf("branch_to address 0x%x\n", (int)func);
				printf("\n\nChange VECMAP and branch to ld boot code...\n");
				for (loop = 0; loop < 0x10000; loop++) ;
				DrvFMC_SetVectorPage(LD_BOOT_CODE_ENTRY);
				spChange(*(uint32_t *)LD_BOOT_CODE_ENTRY);	
				func();
				break;

			case '1':
				func = 	(FUNC_PTR *)(*(uint32_t *)(USER_AP_ENTRY+4));
				printf("branch_to address 0x%x\n", (int)func);
				printf("\n\nChange VECMAP and branch to user application...\n");
				for (loop = 0; loop < 0x10000; loop++) ;
				DrvFMC_SetVectorPage(USER_AP_ENTRY);
				spChange(*(uint32_t *)USER_AP_ENTRY);
				func();
				break;
				
			case '2':
				printf("Loading new user AP image 1 to APROM...\n");
				DrvFMC_EnableAPUpdate();
				if (load_image_to_flash((uint32_t)&ApImage1Base, (uint32_t)&ApImage1Limit, 
										USER_AP_ENTRY, USER_AP_MAX_SIZE) != 0)
				{
					printf("Load image to APROM failed!\n");
					return -1;
				}
				DrvFMC_DisableAPUpdate();
				
				func = 	(FUNC_PTR *)(*(uint32_t *)(USER_AP_ENTRY+4));
				printf("branch_to address 0x%x\n", (int)func);
				printf("\n\nChange VECMAP and branch to user application...\n");
				for (loop = 0; loop < 0x10000; loop++) ;
				DrvFMC_SetVectorPage(0x0);
				func();
				break;

		case '3':
				printf("Loading new user AP image 2 to APROM...\n");
				DrvFMC_EnableAPUpdate();
				if (load_image_to_flash((uint32_t)&ApImage2Base, (uint32_t)&ApImage2Limit, 
										USER_AP_ENTRY, USER_AP_MAX_SIZE) != 0)
				{
					printf("Load image to APROM failed!\n");
					return -1;
				}
				DrvFMC_DisableAPUpdate();
				
				func = 	(FUNC_PTR *)(*(uint32_t *)(USER_AP_ENTRY+4));
				printf("branch_to address 0x%x\n", (int)func);
				printf("\n\nChange VECMAP and branch to user application...\n");
				for (loop = 0; loop < 0x10000; loop++) ;
				DrvFMC_SetVectorPage(0x0);
				func();

			default :
				continue;
		}
    } while (1);



	return 0;
}


static int  load_image_to_flash(uint32_t image_base, uint32_t image_limit, uint32_t flash_addr, uint32_t max_size)
{
	uint32_t   i, j, u32Data, u32ImageSize, *pu32Loader;
	
    u32ImageSize = image_limit - image_base;
    if (u32ImageSize == 0)
    {
        printf("  ERROR: Loader Image is 0 bytes!\n");
        return -1;
    }

    if (u32ImageSize > max_size)
    {
        printf("  ERROR: Loader Image is larger than %d KBytes!\n", max_size/1024);
        return -1;    
    }

    printf("Program image to flash address 0x%x...", flash_addr);
    pu32Loader = (uint32_t *)image_base;
    for (i = 0; i < u32ImageSize; i += PAGE_SIZE)
    {
        if (DrvFMC_Erase(flash_addr + i))
        {
        	printf("Erase failed on 0x%x\n", flash_addr + i);
        	return -1;
        }
        
        for (j = 0; j < PAGE_SIZE; j += 4)
        {
            if (DrvFMC_Write(flash_addr + i + j, pu32Loader[(i + j) / 4]))
            {
            	printf("Write faild on 0x%x\n", flash_addr + i + j);
            	return -1; 
            }
        }
    }
    printf("OK.\n");
    
    printf("Verify ...");

    /* Verify loader */
    for (i = 0; i < u32ImageSize; i += PAGE_SIZE)
    {
        for (j = 0; j < PAGE_SIZE; j += 4)
        {
            if (DrvFMC_Read(flash_addr + i + j, &u32Data))
            {
            	printf("Read failed on 0x%x\n", flash_addr + i + j);
            	return -1;
            }
            if (u32Data != pu32Loader[(i+j)/4])
            {
            	printf("data mismatch on 0x%x, [0x%x], [0x%x]\n", flash_addr + i + j, u32Data, pu32Loader[(i+j)/4]);
            	return -1;  
            }
            
            if (i + j >= u32ImageSize)
                break;
        }
    }
    printf("OK.\n");
    return 0;
}

/*** (C) COPYRIGHT 2012 Nuvoton Technology Corp. ***/
