/******************************************************************************
 * @file     ld_boot.c
 * @brief    LDROM main routine.
 * @version  1.0.0
 * @date     18, October, 2012
 *
 * @note
 * Copyright (C) 2012-2014 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#include <stdio.h>
#include "NUC1xx.h"
#include "Driver\DrvUART.h"
#include "Driver\DrvSYS.h"
#include "Driver\DrvFMC.h"

#define PLLCON_SETTING      SYSCLK_PLLCON_50MHz_XTAL
#define PLL_CLOCK           50000000

#define CONFIG0	0x300000
#define CONFIG1 0x300004

#define USER_AP_ENTRY			(0)
#define IAP_CODE_ENTRY			(50*1024)
//#define IAP_CODE_ENTRY			(0x4000)
 				

typedef void (FUNC_PTR)(void);


__asm spChange(uint32_t _sp)
{
	MSR MSP, r0
	BX lr
}

/*---------------------------------------------------------------------------------------------------------*/
/*  Main Function									                                           			   */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main (void)
{
	STR_UART_T sParam;
	int  			u8Item;
	FUNC_PTR  		*func;
	volatile int  	loop;
	uint32_t	config0, config1;
	int			cbs;
	
	UNLOCKREG();
    SYSCLK->PWRCON.XTL12M_EN = 1;
    while(SYSCLK->CLKSTATUS.XTL12M_STB == 0);
	  
	/* Set UART Pin */
	outpw(&SYS->GPBMFP, inpw(&SYS->GPBMFP) | (0x3<<(0)));
	
	/* UART Setting */
    sParam.u32BaudRate 		= 115200;
    sParam.u8cDataBits 		= DRVUART_DATABITS_8;
    sParam.u8cStopBits 		= DRVUART_STOPBITS_1;
    sParam.u8cParity 		= DRVUART_PARITY_NONE;
    sParam.u8cRxTriggerLevel= DRVUART_FIFO_1BYTES;

	/* Set UART Configuration */
 	if(DrvUART_Open(UART_PORT0,&sParam) != E_SUCCESS)
	{		
		return FALSE;
	}
	
	/* Enable ISP function */
    FMC->ISPCON.ISPEN = 1;

    printf("\n\n");
    printf("+--------------------------------------------------+\n");
    printf("|         LD boot program running on LDROM         |\n");
    printf("+--------------------------------------------------+\n");
	
    do
    {      
    	printf("\n\n\n");
		printf("+-------------------------------------------+\n");
		printf("|               Program Select              |\n");
		printf("+-------------------------------------------|\n");
		printf("| [0] Run IAP program (on APROM at 0x4000)  |\n");
		printf("| [1] Run user application                  |\n");
		printf("| [2] Change CBS to 11b (boot from APROM)   |\n");
		printf("+-------------------------------------------+\n");
		printf("Please select...");
		u8Item = getchar();
		printf("%c\n", u8Item);
		
 		switch (u8Item)
		{
			case '0':
				func = 	(FUNC_PTR *)(*(uint32_t *)(IAP_CODE_ENTRY+4));//50K
				printf("branch_to address 0x%x\n", (int)func);
				printf("\n\nChange VECMAP and branch to IAP code...\n");
				for (loop = 0; loop < 0x10000; loop++) ;
				DrvFMC_SetVectorPage(IAP_CODE_ENTRY);//0x0
				spChange(*(uint32_t *)IAP_CODE_ENTRY);				
				func();
				
				break;

			case '1':
				func = 	(FUNC_PTR *)(*(uint32_t *)(USER_AP_ENTRY+4));//0
				printf("branch_to address 0x%x\n", (int)func);
				printf("\n\nChange VECMAP and branch to user application...\n");
				for (loop = 0; loop < 0x10000; loop++) ;
				DrvFMC_SetVectorPage(0x0);
				spChange(*(uint32_t *)USER_AP_ENTRY);
				func();
				break;

			case '2':
				/*
	 			 *  Modify CBS
	 			 */
    			if ((DrvFMC_Read(CONFIG0, &config0) < 0) || (DrvFMC_Read(CONFIG1, &config1) < 0))
   				{
    				printf("\n\nFailed to read Config!\n\n");
    				break; 
    			}
    			cbs = (config0 >> 6) & 0x3;
    			printf("Config0 = 0x%x, Config1 = 0x%x, CBS=%d\n\n", config0, config1, cbs); 

				{
					DrvFMC_EnableConfigUpdate();
					config0 = (config0 | 0xc0);		/* set CBS to 11b */
					DrvFMC_Erase(CONFIG0);
					DrvFMC_Write(CONFIG0, config0 | 0x2);
					DrvFMC_Write(CONFIG1, config1);
					DrvFMC_DisableConfigUpdate();
				}
				printf("\nExecute chip reset, will boot from ARROM...\n\n");
    			for (loop = 0; loop < 0x10000; loop++) ;
	
				// do chip reset
				DrvSYS_ResetChip();			

			default :
				continue;
		}
    } while (1);

}

/*** (C) COPYRIGHT 2012 Nuvoton Technology Corp. ***/


